# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2019-2026, by Samuel Williams.

module Falcon
	# Pre-defined environments for hosting web applications.
	module Environment
	end
end
